<?php
/**
 * J.E.D.I. | Jerry's Easy Demo Import
 *
 * This is the main driver file for importing the demo content.
 *
 * @package		jedi-apprentice
 * @author		Jerry Simmons <jerry@montereypremier.com>
 * @copyright	2017 Jerry Simmons
 * @license		GPL-2.0+
 **/


if ( ! defined( 'ABSPATH' ) ) { exit; }

/**
 * Require WordPress Dependencies
 **/
require_once( ABSPATH . 'wp-admin/includes/image.php' );
require_once( ABSPATH . 'wp-admin/includes/file.php' );
require_once( ABSPATH . 'wp-admin/includes/media.php' );
require_once( ABSPATH . 'wp-admin/includes/class-pclzip.php' );

/**
 * Define Paths - Includes trailing slash
 **/
define( "JEDI_APPRENTICE_PATH", plugin_dir_path(__FILE__) );
define( "JEDI_APPRENTICE_URL", plugin_dir_url(__FILE__) );


/**
 * Require JEDI Classes
 **/
require_once( JEDI_APPRENTICE_PATH . 'class-jedi-apprentice-admin.php' );
require_once( JEDI_APPRENTICE_PATH . 'class-jedi-apprentice-import.php' );
require_once( JEDI_APPRENTICE_PATH . 'class-jedi-beacon.php' );


/**
 * Initialize JEDI Installer Settings
 *
 * Runs when database value is not set, or when plugin is activated
 **/
register_activation_hook( __FILE__, 'train_jedi_apprentice' );
if( false == get_option('jedi_apprentice_settings') ) { train_jedi_apprentice(); }

// Catch switch to another JEDI theme, and reload settings from file
if( JEDI_APPRENTICE_PATH !== get_option( 'jedi_apprentice_path' ) ) { train_jedi_apprentice(); }

function train_jedi_apprentice() {
	$jedi_beacon = new JEDI_Beacon;
	$jedi_beacon->jedi_log( 'Operating System: ' . PHP_OS . ', PHP Version: ' . PHP_VERSION );

	$jedi_settings_file = JEDI_APPRENTICE_PATH .'/demo-data/jedi_data_export.dat';
	if( file_exists( $jedi_settings_file ) ) {
		$jedi_settings = unserialize( file_get_contents( $jedi_settings_file ) );
	} else {
		$jedi_beacon->order_66( 'Unable to load settings file', $jedi_settings_file );
	}
	if( false === $jedi_settings ) {
		$jedi_beacon->order_66( 'Failed to load installer settings' );
	}

	$jedi_apprentice_settings = $jedi_settings['jedi_settings'];
	$jedi_apprentice_settings['jedi_import_sysinfo'] = $jedi_beacon->get_system_info();
	update_option( 'jedi_apprentice_settings', $jedi_apprentice_settings );
	update_option( 'jedi_import_options', $jedi_apprentice_settings );
	update_option( 'jedi_apprentice_path', JEDI_APPRENTICE_PATH );

	$jedi_beacon->jedi_log( 'Initialize Installer Settings', $jedi_apprentice_settings );
} // END train_jedi_apprentice()

/**
 * Load JEDI Apprentice Admin Menu
 **/
$jedi_apprentice_admin = new JEDI_Apprentice_Admin;